@@ -43,6 +43,9 @@
 
 #include <string>
 #include <fstream>
+#include <regex>
+#include <algorithm>
+#include <vector>
 
 #include <gpgme++/context.h>
 #include <gpgme++/error.h>
@@ -52,12 +55,6 @@
 
 HINSTANCE glob_hinst = NULL;
 
-void
-set_global_hinstance (HINSTANCE hinst)
-{
-    glob_hinst = hinst;
-}
-
 void
 bring_to_front (HWND wid)
 {
@@ -246,6 +243,23 @@ readRegStr (const char *root, const char *dir, const char *name)
     if (!(root_key = get_root_key(root))) {
         return ret;
     }
+    if (root == nullptr)
+      {
+        /* Nullptr so we first look into HKLM if we have
+           an override */
+        ret = _readRegStr (HKEY_LOCAL_MACHINE, dir, name, false);
+        if (ret.empty()) {
+            // Try alternate as fallback
+            ret = _readRegStr (HKEY_LOCAL_MACHINE, dir, name, true);
+        }
+        if (ret.size() && ret[ret.size() - 1] == '!')
+          {
+            // Using override reg value
+            log_dbg ("Using override for %s", name);
+            ret.pop_back();
+            return ret;
+          }
+      }
     ret = _readRegStr (root_key, dir, name, false);
     if (ret.empty()) {
         // Try alternate as fallback
@@ -464,9 +478,25 @@ get_pretty_attachment_name (wchar_t *path, protocol_t protocol,
   return pretty;
 }
 
+BOOL
+DeleteFileUtf8 (const char *utf8Name)
+{
+  SetLastError (0);
+  if (!utf8Name)
+    {
+      STRANGEPOINT;
+      return false;
+    }
+  wchar_t *wname = utf8_to_wchar (utf8Name);
+  BOOL ret = DeleteFileW (wname);
+  xfree (wname);
+  return ret;
+}
+
 HANDLE
 CreateFileUtf8 (const char *utf8Name)
 {
+  SetLastError (0);
   if (!utf8Name)
     {
       return INVALID_HANDLE_VALUE;
@@ -481,7 +511,7 @@ CreateFileUtf8 (const char *utf8Name)
 
   auto ret = CreateFileW (wname,
                           GENERIC_WRITE | GENERIC_READ,
-                          FILE_SHARE_READ | FILE_SHARE_DELETE,
+                          FILE_SHARE_READ,
                           NULL,
                           CREATE_NEW,
                           FILE_ATTRIBUTE_TEMPORARY,
@@ -549,13 +579,14 @@ getTmpPathUtf8 ()
 wchar_t*
 get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
 {
-  const auto utf8Name = wchar_to_utf8_string (name);
+  TSTART
+  auto utf8Name = sanitizeFileName(wchar_to_utf8_string (name));
   const auto tmpPath = getTmpPathUtf8 ();
 
   if (utf8Name.empty() || tmpPath.empty())
     {
       TRACEPOINT;
-      return nullptr;
+      TRETURN nullptr;
     }
 
   auto outName = tmpPath + utf8Name;
@@ -564,11 +595,29 @@ get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
                   SRCNAME, __func__, outName.c_str ());
 
   int tries = 1;
+  bool deletedFile = false;
   while ((*outHandle = CreateFileUtf8 (outName.c_str ())) == INVALID_HANDLE_VALUE)
     {
-      log_debug_w32 (-1, "%s:%s: Failed to open candidate '%s'",
+
+      DWORD err = GetLastError ();
+      log_debug_w32 (err, "%s:%s: Failed to open candidate '%s'",
                      SRCNAME, __func__, anonstr (outName.c_str()));
 
+      if (!deletedFile && err == ERROR_FILE_EXISTS)
+        {
+          /* Try to delete the file. If someone is reading it we should
+             not be able to delete it. */
+          if (DeleteFileUtf8 (outName.c_str ()))
+            {
+              log_dbg ("Deleted existing tmp file '%s'", anonstr (outName.c_str ()));
+              deletedFile = true;
+              continue;
+            }
+        }
+      /* Just to make sure we do not loop endlessly if we get strange return
+       * values. */
+      deletedFile = false;
+
       char *outNameC = xstrdup ((tmpPath + utf8Name).c_str ());
 
       const auto lastBackslash = strrchr (outNameC, '\\');
@@ -578,7 +627,7 @@ get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
           log_error ("%s:%s: No backslash in origname '%s'",
                      SRCNAME, __func__, outNameC);
           xfree (outNameC);
-          return NULL;
+          TRETURN NULL;
         }
 
       auto fileExt = strchr (lastBackslash, '.');
@@ -605,22 +654,33 @@ get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
           log_dbg ("Can't find an attachment name. "
                    "Switching over to a generic attachment name.");
           outName = tmpPath + "attachment";
+          utf8Name = "attachment";
           if (fileExt)
             {
               outName += ".";
               outName += fileExt;
+              utf8Name += ".";
+              utf8Name += fileExt;
             }
         }
       if (tries > 100)
         {
+          char *buf;
+          gpgrt_asprintf (&buf,"Failed to obtain temporary filename '%s'"
+                             " please check that the folder '%s' exists and the file is"
+                             " writable. Look into configuration dialog for debug options.",
+                             wchar_to_utf8_string (name).c_str (), tmpPath.c_str ());
+          memdbg_alloc (buf);
           /* You have to know when to give up,.. */
+          gpgol_message_box (NULL, buf, _("GpgOL Error"), MB_OK);
+          xfree (buf);
           log_error ("%s:%s: Could not get a name out of 100 tries",
                      SRCNAME, __func__);
-          return NULL;
+          TRETURN NULL;
         }
     }
 
-  return utf8_to_wchar (outName.c_str ());
+  TRETURN utf8_to_wchar (outName.c_str ());
 }
 
 char *
@@ -1032,8 +1092,8 @@ de_vs_name (bool isCompliant)
     }
   TSTART;
   /* Only look once */
-  compName = _("VS-NfD compliant");
-  uncompName = _("not VS-NfD compliant");
+  compName = utf8_gettext ("VS-NfD compliant");
+  uncompName = utf8_gettext ("not VS-NfD compliant");
   /* Find the libkleopatrarc */
   char *instdir = get_gpg4win_dir();
   if (!instdir)
@@ -1120,17 +1180,17 @@ compliance_string (bool forVerify, bool forDecrypt, bool isCompliant)
   const char *comp_name = de_vs_name (isCompliant);
   if (forDecrypt && forVerify)
     {
-      return asprintf_s (_("The message is %s."), comp_name);
+      return asprintf_s (utf8_gettext ("The message is %s."), comp_name);
     }
   if (forVerify)
     {
       /* TRANSLATORS %s is compliance name like VS-NfD */
-      return asprintf_s (_("The signature is %s."), comp_name);
+      return asprintf_s (utf8_gettext ("The signature is %s."), comp_name);
     }
   if (forDecrypt)
     {
       /* TRANSLATORS %s is compliance name like VS-NfD */
-      return asprintf_s (_("The encryption is %s."),
+      return asprintf_s (utf8_gettext ("The encryption is %s."),
                             comp_name);
     }
   /* Should not happen */
@@ -1138,3 +1198,125 @@ compliance_string (bool forVerify, bool forDecrypt, bool isCompliant)
   STRANGEPOINT;
   return std::string();
 }
+
+char *
+get_gpgme_w32_inst_dir (void)
+{
+  char *gpg4win_dir = get_gpg4win_dir ();
+  char *tmp;
+  gpgrt_asprintf (&tmp, "%s\\bin\\gpgme-w32spawn.exe", gpg4win_dir);
+  memdbg_alloc (tmp);
+
+  if (!access(tmp, R_OK))
+    {
+      xfree (tmp);
+      gpgrt_asprintf (&tmp, "%s\\bin", gpg4win_dir);
+      memdbg_alloc (tmp);
+      xfree (gpg4win_dir);
+      return tmp;
+    }
+  xfree (tmp);
+  gpgrt_asprintf (&tmp, "%s\\gpgme-w32spawn.exe", gpg4win_dir);
+  memdbg_alloc (tmp);
+
+  if (!access(tmp, R_OK))
+    {
+      xfree (tmp);
+      return gpg4win_dir;
+    }
+  OutputDebugString("Failed to find gpgme-w32spawn.exe!");
+  return NULL;
+}
+
+#define WINDOWS_DEVICES_PATTERN "(CON|AUX|PRN|NUL|COM[1-9]|LPT[1-9])(\\..*)?"
+#define SLASHES "/\\"
+
+static const std::regex&
+windowsDeviceNoSubDirPattern()
+{
+  static const std::regex rc ("^" WINDOWS_DEVICES_PATTERN "$", std::regex_constants::icase);
+  return rc;
+}
+
+static const std::regex&
+windowsDeviceSubDirPattern()
+{
+  static const std::regex rc ("^.*[/\\\\]" WINDOWS_DEVICES_PATTERN "$", std::regex_constants::icase);
+  return rc;
+}
+
+static const char notAllowedCharsSubDir[] = ",^@={}[]~!?:&*\"|#%<>$\"'();`' ";
+static const char notAllowedCharsNoSubDir[] = ",^@={}[]~!?:&*\"|#%<>$\"'();`' " SLASHES;
+static const char* notAllowedSubStrings[] = {".."};
+
+bool
+validateWindowsFileName(const std::string& name, bool allowDirectories)
+{
+  if (name.empty ())
+    {
+      return false;
+    }
+
+  // Characters
+  const char* notAllowedChars = allowDirectories ? notAllowedCharsSubDir
+                                                 : notAllowedCharsNoSubDir;
+  for (const char* c = notAllowedChars; *c; ++c)
+    {
+      if (name.find(*c) != std::string::npos)
+        {
+          return false;
+        }
+    }
+
+  // Substrings
+  for (const auto& subStr : notAllowedSubStrings) {
+      if (name.find(subStr) != std::string::npos)
+        {
+          return false;
+        }
+  }
+
+  // Windows devices
+  bool matchesWinDevice = std::regex_match(name, windowsDeviceNoSubDirPattern());
+  if (!matchesWinDevice && allowDirectories)
+    {
+      matchesWinDevice = std::regex_match(name, windowsDeviceSubDirPattern());
+    }
+  return !matchesWinDevice;
+}
+
+std::string
+sanitizeFileName(std::string name, bool allowDirectories)
+{
+  if (name.empty())
+    {
+      return "attachment";
+    }
+
+  // Characters
+  const char* notAllowedChars = allowDirectories ? notAllowedCharsSubDir
+                                                 : notAllowedCharsNoSubDir;
+  for (const char* c = notAllowedChars; *c; ++c)
+    {
+      name.erase(std::remove(name.begin(), name.end(), *c), name.end());
+    }
+
+  // Substrings
+  for (const auto& subStr : notAllowedSubStrings)
+    {
+      size_t pos;
+      while ((pos = name.find(subStr)) != std::string::npos)
+        {
+          name.erase(pos, std::strlen(subStr));
+        }
+    }
+
+  // Windows devices
+  if (std::regex_match(name, windowsDeviceNoSubDirPattern()) ||
+      (allowDirectories && std::regex_match(name, windowsDeviceSubDirPattern())))
+    {
+      name = "_" + name;
+    }
+
+  return name.empty() ? "_" : name;
+}
